//-----------------------------------------------------------------------------
// File: VertexShader.cpp
//
// Desc: Example code showing how to do vertex shaders in D3D.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#define STRICT
#include <Windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <math.h>
#include <D3DX9.h>
#include "DXUtil.h"
#include "D3DEnumeration.h"
#include "D3DSettings.h"
#include "D3DApp.h"
#include "D3DFile.h"
#include "D3DFont.h"
#include "D3DUtil.h"
#include "resource.h"
#include "const.h"



//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the 
//       generic functionality needed in all Direct3D samples. CMyD3DApplication 
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
class CMyD3DApplication : public CD3DApplication
{
	enum{ MODEL_WOMAN, MODEL_DROID, MODEL_BALL, MODEL_MAX};
	enum{ LINE_NONE, LINE_1PASS, LINE_2PASS, LINE_2SMOOTH, LINE_MAX};
    // Font for drawing text
    CD3DFont* m_pFont;
    CD3DFont* m_pFontSmall;

    // Scene
    LPDIRECT3DVERTEXDECLARATION9 m_pObjectVertexDeclaration;
    LPDIRECT3DVERTEXDECLARATION9 m_pQuadVertexDeclaration;
    LPDIRECT3DVERTEXSHADER9 m_pNormalEdgeVertexShader;
    LPDIRECT3DVERTEXSHADER9 m_pHandEdgeVertexShader;
    LPDIRECT3DVERTEXSHADER9 m_pToonVertexShader;
    LPDIRECT3DVERTEXSHADER9 m_pQuadVertexShader;
	LPDIRECT3DPIXELSHADER9	m_pQuadPixelShader;
	LPDIRECT3DPIXELSHADER9	m_pDilationPixelShader;
	LPDIRECT3DPIXELSHADER9	m_pSmoothDilationPixelShader;
	LPDIRECT3DPIXELSHADER9	m_pSmoothPixelShader;
	LPDIRECT3DPIXELSHADER9	m_pHandEdgePixelShader;

    LPDIRECT3DVERTEXBUFFER9 m_pQuadVB;

    LPDIRECT3DVERTEXBUFFER9 m_pWomanVB;
    LPDIRECT3DINDEXBUFFER9  m_pWomanIB;
	LPD3DXMESH	m_pWomanMesh;
	LPD3DXBUFFER	m_pWomanD3DXMtrlBuffer;
	LPD3DXATTRIBUTERANGE	m_pWomanAttributes;
	DWORD	m_dwWomanNumSections;

    LPDIRECT3DVERTEXBUFFER9 m_pDroidVB;
    LPDIRECT3DINDEXBUFFER9  m_pDroidIB;
	LPD3DXMESH	m_pDroidMesh;
	LPD3DXBUFFER	m_pDroidD3DXMtrlBuffer;
	LPD3DXATTRIBUTERANGE	m_pDroidAttributes;
	DWORD	m_dwDroidNumSections;

    LPDIRECT3DVERTEXBUFFER9 m_pBallVB;
    LPDIRECT3DINDEXBUFFER9  m_pBallIB;
	LPD3DXMESH	m_pBallMesh;
	LPD3DXBUFFER	m_pBallD3DXMtrlBuffer;
	LPD3DXATTRIBUTERANGE	m_pBallAttributes;
	DWORD	m_dwBallNumSections;

	LPDIRECT3DTEXTURE9	m_pShadeTextureA;
	LPDIRECT3DTEXTURE9	m_pWomanTexture;
	LPDIRECT3DTEXTURE9	m_pBallTexture;
	LPDIRECT3DTEXTURE9	m_pLinesTexture;
	LPDIRECT3DCUBETEXTURE9 m_pNormalCubeTexture;
	LPDIRECT3DTEXTURE9	m_pRenderTextureA;
	LPDIRECT3DTEXTURE9	m_pRenderTextureB;
	LPDIRECT3DTEXTURE9	m_pFakeNormalTexture;
	LPDIRECT3DSURFACE9	m_pDepthSurfaceA;
	LPDIRECT3DSURFACE9	m_pRenderSurfaceA;
	LPDIRECT3DSURFACE9	m_pRenderSurfaceB;
	LPDIRECT3DSURFACE9	m_pFakeNormalSurface;
	LPDIRECT3DSURFACE9	m_pBackbufferColor;
	LPDIRECT3DSURFACE9	m_pBackbufferDepth;
	D3DCOLORVALUE	m_colLightCombined;

    // Transforms
    D3DXMATRIXA16  m_matPosition;
    D3DXMATRIXA16  m_matWorld;
    D3DXMATRIXA16  m_matView;
    D3DXMATRIXA16  m_matProj;

    // Navigation
    BYTE        m_bKey[256];
    BYTE        m_bKeyRelease[256];
    FLOAT       m_fSpeed;
    FLOAT       m_fAngularSpeed;
    BOOL        m_bShowHelp;
	BOOL		m_bShading;
	BOOL		m_bDebug;
	BYTE		m_Model;
	BYTE		m_LineStyle;
	FLOAT		m_fLineWidth;

    D3DXVECTOR3 m_vVelocity;
    D3DXVECTOR3 m_vAngularVelocity;

protected:
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT FinalCleanup();
    HRESULT Render();
    HRESULT FrameMove();
    HRESULT ConfirmDevice( D3DCAPS9* pCaps, DWORD dwBehavior, 
		D3DFORMAT adapterFormat, D3DFORMAT backBufferFormat );
    LRESULT MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

public:
    CMyD3DApplication();
};




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    CMyD3DApplication d3dApp;

    InitCommonControls();
    if( FAILED( d3dApp.Create( hInst ) ) )
        return 0;

    return d3dApp.Run();
}




//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
CMyD3DApplication::CMyD3DApplication()
{
    m_strWindowTitle    = _T("Vish Shader");
    m_d3dEnumeration.AppUsesDepthBuffer   = TRUE;

    m_pFont            = new CD3DFont( _T("Arial"), 12, D3DFONT_BOLD );
    m_pFontSmall       = new CD3DFont( _T("Arial"),  9, D3DFONT_BOLD );
    m_pWomanIB              = NULL;
    m_pWomanVB              = NULL;
	m_pWomanMesh			= NULL;
	m_pWomanD3DXMtrlBuffer	= NULL;
	m_pWomanAttributes		= NULL;
    m_pDroidIB              = NULL;
    m_pDroidVB              = NULL;
	m_pDroidMesh			= NULL;
	m_pDroidD3DXMtrlBuffer	= NULL;
	m_pDroidAttributes		= NULL;
    m_pBallIB				= NULL;
    m_pBallVB				= NULL;
	m_pBallMesh				= NULL;
	m_pBallD3DXMtrlBuffer	= NULL;
    m_pQuadVB				= NULL;

    m_pNormalEdgeVertexShader		= NULL;
    m_pHandEdgeVertexShader			= NULL;
    m_pToonVertexShader				= NULL;
    m_pQuadVertexShader				= NULL;

    m_pObjectVertexDeclaration		= NULL;
    m_pQuadVertexDeclaration		= NULL;

    m_pQuadPixelShader				= NULL;
    m_pDilationPixelShader			= NULL;
    m_pSmoothDilationPixelShader	= NULL;
    m_pSmoothPixelShader			= NULL;
    m_pHandEdgePixelShader		= NULL;

	m_pWomanTexture			= NULL;
	m_pBallTexture			= NULL;
	m_pLinesTexture			= NULL;
	m_pShadeTextureA		= NULL;
	m_pNormalCubeTexture	= NULL;
	m_pRenderTextureA		= NULL;
	m_pRenderTextureB		= NULL;
	m_pFakeNormalTexture    = NULL;

	m_pDepthSurfaceA		= NULL;
	m_pRenderSurfaceA		= NULL;
	m_pRenderSurfaceB		= NULL;
	m_pFakeNormalSurface    = NULL;

	m_pBackbufferColor		= NULL;
	m_pBackbufferDepth		= NULL;

    ZeroMemory( m_bKey, sizeof(m_bKey) );
    ZeroMemory( m_bKeyRelease, sizeof(m_bKeyRelease) );
    m_fSpeed           = 100.0f;
    m_fAngularSpeed    = 1.0f;
    m_bShowHelp        = FALSE;
    m_bShading			= TRUE;
    m_bDebug			= FALSE;
    m_Model				= MODEL_WOMAN;
    m_LineStyle			= LINE_1PASS;
	m_fLineWidth		= 1.0f;
    m_vVelocity        = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    m_vAngularVelocity = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
}




//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::OneTimeSceneInit()
{
    
	// Setup the view matrix
    D3DXVECTOR3 vEye = D3DXVECTOR3( 0.0f, 0.0f, -600.0f );
    D3DXVECTOR3 vAt  = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUp  = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &m_matView, &vEye, &vAt, &vUp );

    // Set the position matrix
    D3DXMatrixInverse( &m_matPosition, NULL, &m_matView );

	// Set Quad Matrices.
	D3DXVECTOR3 const vQuadEye	= D3DXVECTOR3( 0.0f, 0.0f, -5.0f );
	D3DXVECTOR3 const vQuadAt	= D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 const vQuadUp	= D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FrameMove()
{
    FLOAT fSecsPerFrame = m_fElapsedTime;

    // Process keyboard input
    D3DXVECTOR3 vT( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vR( 0.0f, 0.0f, 0.0f );

    if( m_bKey[VK_LEFT] || m_bKey[VK_NUMPAD1] )                 vT.x -= 1.0f; // Slide Left
    if( m_bKey[VK_RIGHT] || m_bKey[VK_NUMPAD3] )                vT.x += 1.0f; // Slide Right
    if( m_bKey[VK_DOWN] )                                       vT.y -= 1.0f; // Slide Down
    if( m_bKey[VK_UP] )                                         vT.y += 1.0f; // Slide Up
    if( m_bKey['W'] )                                           vT.z += 2.0f; // Move Forward
    if( m_bKey['S'] )                                           vT.z -= 2.0f; // Move Backward
    if( m_bKey['D'] || m_bKey[VK_NUMPAD6] )                     vR.y += 1.0f; // Turn Right
    if( m_bKey['A'] || m_bKey[VK_NUMPAD4] )                     vR.y -= 1.0f; // Turn Left
    if( m_bKey['F'] || m_bKey[VK_NUMPAD8] )                     vR.x -= 1.0f; // Pitch Down
    if( m_bKey['R'] || m_bKey[VK_NUMPAD2] )                     vR.x += 1.0f; // Pitch Up
    if( m_bKey[VK_NUMPAD9] )                                    vR.z += 2.0f; // Roll CW
    if( m_bKey[VK_NUMPAD7] )                                    vR.z -= 2.0f; // Roll CCW

	// Release Toggles
    if( m_bKeyRelease['1'] )
	{
		m_LineStyle++; // toggle outline
		if(m_LineStyle == LINE_MAX)
		{
			m_LineStyle = 0;
		}
		m_bKeyRelease['1'] = 0;
	}
    if( m_bKeyRelease['2'] )
	{
		m_bShading = !m_bShading; // toggle shading
		m_bKeyRelease['2'] = 0;
	}
    if( m_bKeyRelease['3'] )
	{
		m_Model++; // switch model
		if(m_Model == MODEL_MAX)
		{
			m_Model = 0;
		}
		m_bKeyRelease['3'] = 0;
	}
    if( m_bKeyRelease['6'] )
	{
		m_bDebug = !m_bDebug; // show line input
		m_bKeyRelease['6'] = 0;
	}

	BOOL bNewWidth = false;
    if( m_bKeyRelease['9'] )
	{
		m_fLineWidth -= 0.1f;
		if(m_fLineWidth < 0.5f)
			m_fLineWidth = 0.5f;
		m_bKeyRelease['9'] = 0;
		bNewWidth = true;
	}
    if( m_bKeyRelease['0'] )
	{
		m_fLineWidth += 0.1f;
		if(m_fLineWidth > 2.0f)
			m_fLineWidth = 2.0f;

		m_bKeyRelease['0'] = 0;
		bNewWidth = true;
	}
	if(bNewWidth)
	{
		SAFE_RELEASE( m_pRenderTextureA );
		SAFE_RELEASE( m_pRenderSurfaceA );
		SAFE_RELEASE( m_pDepthSurfaceA );
		// Create depth buffer surface
		HRESULT hr = m_pd3dDevice->CreateDepthStencilSurface(static_cast<int>(m_d3dsdBackBuffer.Width / m_fLineWidth), static_cast<int>(m_d3dsdBackBuffer.Height / m_fLineWidth), D3DFMT_D24X8, D3DMULTISAMPLE_NONE, 0, FALSE, &m_pDepthSurfaceA, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create depth surface.", "VishShader.exe", MB_OK);
			return hr;
		}

		// Create render target texture
		hr = m_pd3dDevice->CreateTexture(static_cast<int>(m_d3dsdBackBuffer.Width / m_fLineWidth), static_cast<int>(m_d3dsdBackBuffer.Height / m_fLineWidth), 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &m_pRenderTextureA, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create render texture.", "VishShader.exe", MB_OK);
			return hr;
		}
		// Grab the associated surface
		m_pRenderTextureA->GetSurfaceLevel(0, &m_pRenderSurfaceA);
		
		// Fix sampler offsets
		// first shader size changes based on LineWidth
		float const     kPerTexelWidth  = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Width / m_fLineWidth);
		float const     kPerTexelHeight = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Height / m_fLineWidth);
		float           s               = 0.5f;

		// second offset uses shader thats always the size of backbuffer
		float const     kPerTexelWidth2  = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Width);
		float const     kPerTexelHeight2 = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Height);
		float           s2               = 0.5f;
		float const     rotAngle1       = D3DXToRadian( 180.0f );
		float const     rotAngle2       = rotAngle1 + D3DXToRadian(120.0f); 
		float const     rotAngle3       = rotAngle1 + D3DXToRadian(240.0f); 


		float const OffsetX[4] = {	-s * kPerTexelWidth, 
                                    -s * kPerTexelWidth,  
                                     s * kPerTexelWidth,   
                                     s * kPerTexelWidth  };
		float const OffsetY[4] = {	-s * kPerTexelHeight, 
                                     s * kPerTexelHeight, 
                                     s * kPerTexelHeight, 
                                    -s * kPerTexelHeight };

		float const OffsetBX[4] = {	-s2 * kPerTexelWidth2, 
                                    -s2 * kPerTexelWidth2,  
                                     s2 * kPerTexelWidth2,   
                                     s2 * kPerTexelWidth2  };
		float const OffsetBY[4] = {	-s2 * kPerTexelHeight2, 
                                     s2 * kPerTexelHeight2, 
                                     s2 * kPerTexelHeight2, 
                                    -s2 * kPerTexelHeight2 };
		for (int i = 0; i < 4; ++i)
		{
			D3DXVECTOR4  Offset(OffsetX[i], OffsetY[i], 0.0f, 0.0f);
			D3DXVECTOR4  OffsetB(OffsetBX[i], OffsetBY[i], 0.0f, 0.0f);
    		m_pd3dDevice->SetVertexShaderConstantF(TEXTURE_OFFSET + i, (float *)&Offset, 1);
    		m_pd3dDevice->SetVertexShaderConstantF(TEXTURE_OFFSET_B + i, (float *)&OffsetB, 1);
		}
  }

    m_vVelocity        = m_vVelocity * 0.9f + vT * 0.1f;
    m_vAngularVelocity = m_vAngularVelocity * 0.9f + vR * 0.1f;

	D3DXMatrixIdentity(&m_matWorld);

	// Update position and view matricies
    D3DXMATRIXA16     matT, matR;
    D3DXQUATERNION qR;

    vT = m_vVelocity * fSecsPerFrame * m_fSpeed;
    vR = m_vAngularVelocity * fSecsPerFrame * m_fAngularSpeed;

    D3DXMatrixTranslation( &matT, vT.x, vT.y, vT.z);
    D3DXMatrixMultiply( &m_matPosition, &matT, &m_matPosition );

    D3DXQuaternionRotationYawPitchRoll( &qR, vR.y, vR.x, vR.z );
    D3DXMatrixRotationQuaternion( &matR, &qR );

    D3DXMatrixMultiply( &m_matPosition, &matR, &m_matPosition );
    D3DXMatrixInverse( &m_matView, NULL, &m_matPosition );

    // Set up the shader constants
    {
		// set the clip matrix
		m_pd3dDevice->SetVertexShaderConstantF(CLIP_MATRIX, (float *)&(m_matWorld *
												m_matView * m_matProj), 4);

		// transpose for normal transformation
		D3DXMATRIXA16 matWorldInverse;
//		D3DXMatrixTranspose(&matWorldInverse, &m_matWorld);
		D3DXMatrixInverse(&matWorldInverse, NULL, &m_matWorld);
		m_pd3dDevice->SetVertexShaderConstantF(INVERSE_WORLD_MATRIX,
												(float *)&matWorldInverse,3);
		// eye position
		D3DXVECTOR3 vEyePosition;
		D3DXVec3TransformCoord(&vEyePosition, &D3DXVECTOR3(0,0,0), &m_matPosition);
		m_pd3dDevice->SetVertexShaderConstantF(EYE_POSITION, (float *)&vEyePosition,1);


		D3DXVECTOR3 vEyeNormal;
		D3DXVec3Normalize(&vEyeNormal, &vEyePosition);

		// calculate fake normal coordinate system matrix
		// needs to be calculated per object for best quality,
		// but might work using a point an arbitrary distance in front of the camera
		D3DXMATRIXA16 matNormalCoord;
		D3DXVECTOR3 vAt  = D3DXVECTOR3( 0.0f, 0.0f, 0.0f ); // at should be object center
		D3DXVECTOR3 vUp  = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
		D3DXMatrixLookAtLH( &matNormalCoord, &vEyePosition, &vAt, &vUp );
		// Transpose the matrix because we are going from the Normal coordinate system back into world space
		D3DXMatrixTranspose( &matNormalCoord, &matNormalCoord);

		D3DXVECTOR3 vAway1  = D3DXVECTOR3( 0.62251f, 0.0f, 0.78261f );
		D3DXVECTOR3 vAway2  = D3DXVECTOR3( -0.62251f, 0.0f, 0.78261f );
		D3DXVECTOR3 vAway3  = D3DXVECTOR3( 0.0f, 0.62251f, 0.78261f );
		D3DXVECTOR3 vAway4  = D3DXVECTOR3( 0.0f, -0.62251f, 0.78261f );

		D3DXVec3TransformNormal(&vAway1, &vAway1, &matNormalCoord);
		D3DXVec3TransformNormal(&vAway2, &vAway2, &matNormalCoord);
		D3DXVec3TransformNormal(&vAway3, &vAway3, &matNormalCoord);
		D3DXVec3TransformNormal(&vAway4, &vAway4, &matNormalCoord);

		D3DCOLOR FakeNormalColors[4];
		FakeNormalColors[0] = D3DCOLOR_COLORVALUE(	vAway1.x * 0.5f + 0.5f,
													vAway1.y * 0.5f + 0.5f,
													vAway1.z * 0.5f + 0.5f,
													1.0f );
		FakeNormalColors[1] = D3DCOLOR_COLORVALUE(	vAway2.x * 0.5f + 0.5f,
													vAway2.y * 0.5f + 0.5f,
													vAway2.z * 0.5f + 0.5f,
													1.0f );
		FakeNormalColors[2] = D3DCOLOR_COLORVALUE(	vAway3.x * 0.5f + 0.5f,
													vAway3.y * 0.5f + 0.5f,
													vAway3.z * 0.5f + 0.5f,
													1.0f );
		FakeNormalColors[3] = D3DCOLOR_COLORVALUE(	vAway4.x * 0.5f + 0.5f,
													vAway4.y * 0.5f + 0.5f,
													vAway4.z * 0.5f + 0.5f,
													1.0f );
		D3DCOLOR * pColors;
		// Fill in fake normal texture
		D3DLOCKED_RECT recFakeNormal;
		m_pFakeNormalSurface->LockRect(&recFakeNormal, NULL, D3DLOCK_DISCARD);
		pColors = (D3DCOLOR *)recFakeNormal.pBits;
		pColors[0] = FakeNormalColors[0];
		pColors[1] = FakeNormalColors[1];
		pColors = (D3DCOLOR *)((char *)recFakeNormal.pBits + recFakeNormal.Pitch);
		pColors[0] = FakeNormalColors[2];
		pColors[1] = FakeNormalColors[3];
		m_pFakeNormalSurface->UnlockRect();


		// light position 
		D3DXVECTOR3 vLightPositionA(1.0f, 1.0f, -1.0f);
		D3DXVec3Normalize(&vLightPositionA, &vLightPositionA);
		D3DXVECTOR3 vLightPositionB(-1.0f, 1.0f, 0.0f);
		D3DXVec3Normalize(&vLightPositionB, &vLightPositionB);

//		D3DXMATRIXA16 matLightRotA;
//		D3DXMatrixRotationY( &matLightRotA, -(timeGetTime()/777.0f) );
//		D3DXVec3TransformNormal(&vLightPositionA, &vLightPositionA, &matLightRotA);

		D3DXMATRIXA16 matLightRotB;
		D3DXMatrixRotationY( &matLightRotB, (timeGetTime()/1500.0f) );
		D3DXVec3TransformNormal(&vLightPositionB, &vLightPositionB, &matLightRotB);

		float fColorFactorA = max(0.0f, (min(1.0f, (D3DXVec3Dot(&vLightPositionA, &vEyeNormal) + 0.5f) )) );
		float fColorFactorB = max(0.0f, (min(1.0f, (D3DXVec3Dot(&vLightPositionB, &vEyeNormal) + 0.5f) )) );

		float fToOne = (timeGetTime() % 8000)/4000.0f - 1.0f;
		if (fToOne < 0)
			fToOne = -fToOne;
		fToOne = fToOne * 0.4f + 0.6f;

		D3DCOLORVALUE colLightA = {fToOne, fToOne, fToOne, 1.0};
		float fBrightA = max(colLightA.r, max(colLightA.g, colLightA.b));
		D3DCOLORVALUE colLightB = {0.0f, 0.0f, 0.5f, 1.0f};
		float fBrightB = max(colLightB.r, max(colLightB.g, colLightB.b));

/*		// experimental ways of combining lights
		// overriding light
		m_colLightCombined.r = max(colLightA.r * fColorFactorA, colLightB.r * fColorFactorB);
		m_colLightCombined.g = max(colLightA.g * fColorFactorA, colLightB.g * fColorFactorB);
		m_colLightCombined.b = max(colLightA.b * fColorFactorA, colLightB.b * fColorFactorB);

		// additive colors but not brightness
		float fBrightCombined = max(fBrightA * fColorFactorA, fBrightB * fColorFactorB);
		m_colLightCombined.r = (colLightA.r * fColorFactorA + colLightB.r * fColorFactorB);
		m_colLightCombined.g = (colLightA.g * fColorFactorA + colLightB.g * fColorFactorB);
		m_colLightCombined.b = (colLightA.b * fColorFactorA + colLightB.b * fColorFactorB);
		float fScale = fBrightCombined / max(m_colLightCombined.r, max(m_colLightCombined.g, m_colLightCombined.b));
		m_colLightCombined.r *= fScale;
		m_colLightCombined.g *= fScale;
		m_colLightCombined.b *= fScale;
*/

		// additive light
		m_colLightCombined.r = max(0.0f, (min(1.0f, colLightA.r * fColorFactorA + colLightB.r * fColorFactorB)) );
		m_colLightCombined.g = max(0.0f, (min(1.0f, colLightA.g * fColorFactorA + colLightB.g * fColorFactorB)) );
		m_colLightCombined.b = max(0.0f, (min(1.0f, colLightA.b * fColorFactorA + colLightB.b * fColorFactorB)) );

		m_pd3dDevice->SetPixelShaderConstantF(LIGHT_COLOR, (float *)&m_colLightCombined, 1);

		D3DXVec3Scale(&vLightPositionA, &vLightPositionA, fBrightA);
		D3DXVec3Scale(&vLightPositionB, &vLightPositionB, fBrightB);

		m_pd3dDevice->SetVertexShaderConstantF(LIGHT_POSITION_A, (float *)&vLightPositionA,1);
		m_pd3dDevice->SetVertexShaderConstantF(LIGHT_POSITION_B, (float *)&vLightPositionB,1);

		// set the main scene transpose matrix
		D3DXMATRIXA16 matTrans;
		D3DXMatrixTranspose(&matTrans, &(m_matWorld * m_matView * m_matProj));
		m_pd3dDevice->SetVertexShaderConstantF(TRANS_MATRIX, (float *)&matTrans, 4);

	}


    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::Render()
{
	m_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);

    m_pd3dDevice->SetRenderState(D3DRS_ALPHAREF, (DWORD)0x00000001);
    m_pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE); 
    m_pd3dDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);

    // Clear the backbuffer
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
                         0x00FFFFFF, 1.0f, 0L );

	DWORD dwNumSections;
	D3DXMATERIAL* d3dxMaterials;
	LPD3DXATTRIBUTERANGE pAttributes;

	if(m_Model == MODEL_WOMAN)
	{
		dwNumSections = m_dwWomanNumSections;
		m_pd3dDevice->SetStreamSource( 0, m_pWomanVB, 0, sizeof(D3DXVECTOR3) * 2 + sizeof(D3DCOLOR) + sizeof(D3DXVECTOR2));
		m_pd3dDevice->SetIndices( m_pWomanIB );
		d3dxMaterials = (D3DXMATERIAL*)m_pWomanD3DXMtrlBuffer->GetBufferPointer();
		pAttributes = m_pWomanAttributes;
	}
	else if (m_Model == MODEL_DROID)
	{
		dwNumSections = m_dwDroidNumSections;
		m_pd3dDevice->SetStreamSource( 0, m_pDroidVB, 0, sizeof(D3DXVECTOR3) * 2 + sizeof(D3DCOLOR) + sizeof(D3DXVECTOR2));
		m_pd3dDevice->SetIndices( m_pDroidIB );
		d3dxMaterials = (D3DXMATERIAL*)m_pDroidD3DXMtrlBuffer->GetBufferPointer();
		pAttributes = m_pDroidAttributes;
	}
	else
	{
		dwNumSections = m_dwBallNumSections;
		m_pd3dDevice->SetStreamSource( 0, m_pBallVB, 0, sizeof(D3DXVECTOR3) * 2 + sizeof(D3DCOLOR) + sizeof(D3DXVECTOR2));
		m_pd3dDevice->SetIndices( m_pBallIB );
		d3dxMaterials = (D3DXMATERIAL*)m_pBallD3DXMtrlBuffer->GetBufferPointer();
		pAttributes = m_pBallAttributes;
	}
    // Begin the Shaded Scene
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
		m_pd3dDevice->SetVertexDeclaration( m_pObjectVertexDeclaration );
		m_pd3dDevice->SetVertexShader( m_pToonVertexShader );
		// Draw The Scene
		if(m_bShading)
		{
			m_pd3dDevice->SetPixelShader( m_pSmoothPixelShader );
			m_pd3dDevice->SetTexture(0, m_pNormalCubeTexture);
			m_pd3dDevice->SetTexture(1, NULL);
			m_pd3dDevice->SetTexture(2, m_pShadeTextureA);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
		
			for( DWORD i = 0; i < dwNumSections; i++)
			{
				if(i == 0 && m_Model == MODEL_WOMAN)
				{
					m_pd3dDevice->SetTexture(3, m_pWomanTexture);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_COLORARG1, D3DTA_CURRENT);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_COLOROP, D3DTOP_MODULATE);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_COLORARG2, D3DTA_TEXTURE);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
					D3DCOLORVALUE sectionColor = {	0.0f,
													0.0f,
													0.0f,
													0.0f	};
					m_pd3dDevice->SetPixelShaderConstantF(MATERIAL_DIFFUSE, (float *)&sectionColor, 1);
				} else if(i == 0 && m_Model == MODEL_BALL)
				{
					m_pd3dDevice->SetTexture(3, m_pBallTexture);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_COLORARG1, D3DTA_CURRENT);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_COLOROP, D3DTOP_MODULATE);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_COLORARG2, D3DTA_TEXTURE);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
					m_pd3dDevice->SetTextureStageState(3, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
					m_pd3dDevice->SetSamplerState(3, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
					D3DCOLORVALUE sectionColor = {	0.0f,
													0.0f,
													0.0f,
													0.0f	};
					m_pd3dDevice->SetPixelShaderConstantF(MATERIAL_DIFFUSE, (float *)&sectionColor, 1);
				}
				else
				{

					m_pd3dDevice->SetTextureStageState(3, D3DTSS_COLOROP, D3DTOP_DISABLE);
					m_pd3dDevice->SetTexture(3, NULL);
					D3DCOLORVALUE sectionColor = {	d3dxMaterials[i].MatD3D.Diffuse.r,
													d3dxMaterials[i].MatD3D.Diffuse.g,
													d3dxMaterials[i].MatD3D.Diffuse.b,
													d3dxMaterials[i].MatD3D.Diffuse.a	};
					m_pd3dDevice->SetPixelShaderConstantF(MATERIAL_DIFFUSE, (float *)&sectionColor, 1);
				}

				m_pd3dDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, pAttributes[i].VertexStart, pAttributes[i].VertexCount,
													pAttributes[i].FaceStart * 3, pAttributes[i].FaceCount );

			}
		}
        // End the scene.
        m_pd3dDevice->EndScene();
    }

	if(m_LineStyle != LINE_NONE)
	{

		// Select render texture A
		m_pd3dDevice->SetDepthStencilSurface(m_pDepthSurfaceA);
		m_pd3dDevice->SetRenderTarget(0, m_pRenderSurfaceA);

		// Clear the texture
		m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
							 0x00FFFFFF, 1.0f, 0L );


		// Begin the texture scene
		if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
		{
			if(m_Model == MODEL_BALL)
				m_pd3dDevice->SetVertexShader( m_pHandEdgeVertexShader );
			else
				m_pd3dDevice->SetVertexShader( m_pNormalEdgeVertexShader );

			m_pd3dDevice->SetPixelShader( NULL );
			m_pd3dDevice->SetTexture(0, m_pNormalCubeTexture);
			m_pd3dDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
			m_pd3dDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
			m_pd3dDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE);
			m_pd3dDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
			m_pd3dDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
			m_pd3dDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
			m_pd3dDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
		
			for( DWORD i = 0; i < dwNumSections; i++)
			{
					if(i == 0 && m_Model == MODEL_WOMAN)
					{
// code for grabbing alpha from second stage
						m_pd3dDevice->SetTexture(1, m_pWomanTexture);
						m_pd3dDevice->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
						m_pd3dDevice->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
						m_pd3dDevice->SetSamplerState(1, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
						m_pd3dDevice->SetSamplerState(1, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
						m_pd3dDevice->SetSamplerState(1, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

// following code unnecessary when hand edge pixel shader active
						m_pd3dDevice->SetTextureStageState(1, D3DTSS_COLORARG1, D3DTA_CURRENT);
						m_pd3dDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
						m_pd3dDevice->SetTextureStageState(1, D3DTSS_ALPHAARG1, D3DTA_CURRENT);
						m_pd3dDevice->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
						m_pd3dDevice->SetTextureStageState(1, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
// end following

					}
					else if(i == 0 && m_Model == MODEL_BALL)
					{
						m_pd3dDevice->SetTexture(1, m_pBallTexture); // need this to have alpha values for distance edges
						m_pd3dDevice->SetVertexShader( m_pNormalEdgeVertexShader );
						m_pd3dDevice->SetPixelShader( m_pHandEdgePixelShader );
						m_pd3dDevice->SetTexture(2, m_pLinesTexture);
						m_pd3dDevice->SetSamplerState(2, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
						m_pd3dDevice->SetSamplerState(2, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
						m_pd3dDevice->SetSamplerState(2, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
						m_pd3dDevice->SetSamplerState(2, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
						m_pd3dDevice->SetSamplerState(2, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
						m_pd3dDevice->SetTexture(3, m_pFakeNormalTexture);
						m_pd3dDevice->SetSamplerState(3, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
						m_pd3dDevice->SetSamplerState(3, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
						m_pd3dDevice->SetSamplerState(3, D3DSAMP_MINFILTER, D3DTEXF_POINT);
						m_pd3dDevice->SetSamplerState(3, D3DSAMP_MAGFILTER, D3DTEXF_POINT);
						m_pd3dDevice->SetSamplerState(3, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
					}
					else if(i == 1 && m_Model == MODEL_BALL)
					{
						m_pd3dDevice->SetVertexShader( m_pHandEdgeVertexShader );
					}
					else
					{
						m_pd3dDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
						m_pd3dDevice->SetTexture(1, NULL);
					}
				m_pd3dDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, pAttributes[i].VertexStart, pAttributes[i].VertexCount,
													pAttributes[i].FaceStart * 3, pAttributes[i].FaceCount );
				m_pd3dDevice->SetPixelShader( NULL );

			}
			// End the scene.
			m_pd3dDevice->EndScene();
		}

		// set quad states
		{
			D3DTEXTUREFILTERTYPE filter = D3DTEXF_LINEAR;
			m_pd3dDevice->SetSamplerState(0, D3DSAMP_MINFILTER, filter);
			m_pd3dDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, filter);
			m_pd3dDevice->SetSamplerState(1, D3DSAMP_MINFILTER, filter);
			m_pd3dDevice->SetSamplerState(1, D3DSAMP_MAGFILTER, filter);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_MINFILTER, filter);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_MAGFILTER, filter);
			m_pd3dDevice->SetSamplerState(3, D3DSAMP_MINFILTER, filter);
			m_pd3dDevice->SetSamplerState(3, D3DSAMP_MAGFILTER, filter);
			m_pd3dDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
			m_pd3dDevice->SetSamplerState(1, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(1, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(1, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(2, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
			m_pd3dDevice->SetSamplerState(3, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(3, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);
			m_pd3dDevice->SetSamplerState(3, D3DSAMP_MIPFILTER, D3DTEXF_NONE);

		}

		if(m_LineStyle == LINE_1PASS)
		{
			m_pd3dDevice->SetDepthStencilSurface(m_pBackbufferDepth);
			m_pd3dDevice->SetRenderTarget(0, m_pBackbufferColor);
			// Enable alpha blending.
			m_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

			// Set the source blend state.
			m_pd3dDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);

			// Set the destination blend state.
			m_pd3dDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

		}
		else
		{
			// Select render texture B
			m_pd3dDevice->SetDepthStencilSurface(NULL);
			m_pd3dDevice->SetRenderTarget(0, m_pRenderSurfaceB);

			// Clear the texture
			m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET,
								 0x00FFFFFF, 1.0f, 0L );
		}

		// Begin the texture scene
		if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
		{
			// Set textures
			m_pd3dDevice->SetTexture(0, m_pRenderTextureA);
			m_pd3dDevice->SetTexture(1, m_pRenderTextureA);
			m_pd3dDevice->SetTexture(2, m_pRenderTextureA);
			m_pd3dDevice->SetTexture(3, m_pRenderTextureA);

			// Draw the quad

			m_pd3dDevice->SetVertexDeclaration( m_pQuadVertexDeclaration );
			m_pd3dDevice->SetVertexShader( m_pQuadVertexShader );
			m_pd3dDevice->SetStreamSource( 0, m_pQuadVB, 0, sizeof(D3DXVECTOR3) + sizeof(D3DXVECTOR2));

			if(m_bDebug)
			{
				m_pd3dDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
				m_pd3dDevice->SetTexture(1, NULL);
				m_pd3dDevice->SetTexture(2, NULL);
				m_pd3dDevice->SetTexture(3, NULL);

				// Need new offset
				float fOffsetSet = TEXTURE_OFFSET_B;
				m_pd3dDevice->SetVertexShaderConstantF(OFFSET_SET, (float *)&fOffsetSet, 1);
			}
			else
			{
				m_pd3dDevice->SetPixelShader( m_pQuadPixelShader );
				// use A offset
				float fOffsetSet = TEXTURE_OFFSET;
				m_pd3dDevice->SetVertexShaderConstantF(OFFSET_SET, (float *)&fOffsetSet, 1);
			}
			m_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);


			// End the scene.
			m_pd3dDevice->EndScene();


		}

		if(m_LineStyle != LINE_1PASS)
		{

			// Restore the backbuffer
			m_pd3dDevice->SetDepthStencilSurface(m_pBackbufferDepth);
			m_pd3dDevice->SetRenderTarget(0, m_pBackbufferColor);

			if(m_fLineWidth != 1.0f)
			{
				D3DTEXTUREFILTERTYPE filter = D3DTEXF_PYRAMIDALQUAD;
				m_pd3dDevice->SetSamplerState(0, D3DSAMP_MINFILTER, filter);
				m_pd3dDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, filter);
				m_pd3dDevice->SetSamplerState(1, D3DSAMP_MINFILTER, filter);
				m_pd3dDevice->SetSamplerState(1, D3DSAMP_MAGFILTER, filter);
				m_pd3dDevice->SetSamplerState(2, D3DSAMP_MINFILTER, filter);
				m_pd3dDevice->SetSamplerState(2, D3DSAMP_MAGFILTER, filter);
				m_pd3dDevice->SetSamplerState(3, D3DSAMP_MINFILTER, filter);
				m_pd3dDevice->SetSamplerState(3, D3DSAMP_MAGFILTER, filter);
			}

			// Begin the final scene
			if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
			{

				m_pd3dDevice->SetTexture(0, m_pRenderTextureB);
				m_pd3dDevice->SetTexture(1, m_pRenderTextureB);
				m_pd3dDevice->SetTexture(2, m_pRenderTextureB);
				m_pd3dDevice->SetTexture(3, m_pRenderTextureB);

				if(m_bDebug)
				{
					// offset needs new
					float fOffsetSet = TEXTURE_OFFSET_B;
					m_pd3dDevice->SetVertexShaderConstantF(OFFSET_SET, (float *)&fOffsetSet, 1);
					m_pd3dDevice->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
				}
				else
				{
					// offset B
					float fOffsetSet = TEXTURE_OFFSET_B;
					m_pd3dDevice->SetVertexShaderConstantF(OFFSET_SET, (float *)&fOffsetSet, 1);
					if(m_LineStyle == LINE_2SMOOTH)
					{
						m_pd3dDevice->SetPixelShader( m_pSmoothDilationPixelShader );
					}
					else
					{
						m_pd3dDevice->SetPixelShader( m_pDilationPixelShader );
					}

					// Enable alpha blending.
					m_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

					// Set the source blend state.
					m_pd3dDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);

					// Set the destination blend state.
					m_pd3dDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
				}

				m_pd3dDevice->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);


				// End the scene.
				m_pd3dDevice->EndScene();
			}
		}

	}

    // Begin the Text scene
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
		// Output statistics
		m_pFont->DrawText( 2,  0, D3DCOLOR_ARGB(255,0,0,128), m_strFrameStats );
		m_pFont->DrawText( 2, 20, D3DCOLOR_ARGB(255,0,0,128), m_strDeviceStats );

		if( m_bShowHelp )
		{
			m_pFontSmall->DrawText(  2, 40, D3DCOLOR_ARGB(255,0,0,128),
									_T("Keyboard controls:") );
			m_pFontSmall->DrawText( 20, 60, D3DCOLOR_ARGB(255,0,0,128),
									_T("Move\nTurn\nPitch\nSlide\n")
									_T("Cycle Line Style\nCycle Shading Style\nCycle Model\n")
									_T("Help\nChange device\nExit"));
			m_pFontSmall->DrawText( 210, 60, D3DCOLOR_ARGB(255,0,0,128),
									_T("W,S\nA,D\nR,F\nArrow keys\n")
									_T("1\n2\n3\n")
									_T("F1\nF2\nEsc") );
		}
		else
		{
			m_pFontSmall->DrawText(  2, 40, D3DCOLOR_ARGB(255,0,0,128), 
							   _T("Press F1 for help") );
		}
		// End the scene.
		m_pd3dDevice->EndScene();
	}

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InitDeviceObjects()
{

    HRESULT hr;

    m_pFont->InitDeviceObjects( m_pd3dDevice );
    m_pFontSmall->InitDeviceObjects( m_pd3dDevice );

    // Create normal edge vertex shader
    {
        LPD3DXBUFFER pCode;

        D3DVERTEXELEMENT9 decl[] =
        {
            { 0, 0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
			{ 0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL, 0 },
			{ 0, 24, D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR, 0 },
			{ 0, 28, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
           D3DDECL_END()
        };

        if( FAILED( hr = m_pd3dDevice->CreateVertexDeclaration( decl, &m_pObjectVertexDeclaration ) ) )
        {
			MessageBox(NULL, "Failed to create object declaration.", "VishShader.exe", MB_OK);
            return hr;
        }

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the vertex shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("vertex_shaders\\normal.vsh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble normal shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the vertex shader
        hr = m_pd3dDevice->CreateVertexShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pNormalEdgeVertexShader );
        pCode->Release();
        if( FAILED(hr) )
		{
            return hr;
			MessageBox(NULL, "Failed to create normal shader.", "VishShader.exe", MB_OK);
		}
    }

    // Create hand edge vertex shader
    {
        LPD3DXBUFFER pCode;

		// uses same vertex declaration as Normal shader

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the vertex shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("vertex_shaders\\hand_edge.vsh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble hand edge shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the vertex shader
        hr = m_pd3dDevice->CreateVertexShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pHandEdgeVertexShader );
        pCode->Release();
        if( FAILED(hr) )
		{
            return hr;
			MessageBox(NULL, "Failed to create hand edge shader.", "VishShader.exe", MB_OK);
		}
    }

    // Create toon vertex shader
    {
        LPD3DXBUFFER pCode;

		// uses same vertex declaration as Normal shader

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the vertex shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("vertex_shaders\\toon.vsh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble toon shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the vertex shader
        hr = m_pd3dDevice->CreateVertexShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pToonVertexShader );
        pCode->Release();
        if( FAILED(hr) )
		{
            return hr;
			MessageBox(NULL, "Failed to create toon shader.", "VishShader.exe", MB_OK);
		}
    }

    // Create quad vertex shader
    {
        LPD3DXBUFFER pCode;

        D3DVERTEXELEMENT9 decl[] =
        {
            { 0, 0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
			{ 0, 12, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
           D3DDECL_END()
        };

        if( FAILED( hr = m_pd3dDevice->CreateVertexDeclaration( decl, &m_pQuadVertexDeclaration ) ) )
        {
			MessageBox(NULL, "Failed to create quad declaration.", "VishShader.exe", MB_OK);
            return hr;
        }

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the vertex shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("vertex_shaders\\quad.vsh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble quad shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the vertex shader
        hr = m_pd3dDevice->CreateVertexShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pQuadVertexShader );
        pCode->Release();
        if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create quad shader.", "VishShader.exe", MB_OK);
            return hr;
		}
		
	}

    // Create quad pixel shader
    {
        LPD3DXBUFFER pCode;

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the pixel shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("pixel_shaders\\edge_depth.psh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble quad pixel shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the pixel shader
        hr = m_pd3dDevice->CreatePixelShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pQuadPixelShader );
        pCode->Release();
        if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create quad pixel shader.", "VishShader.exe", MB_OK);
            return hr;
		}
		
	}

    // Create dilation pixel shader
    {
        LPD3DXBUFFER pCode;

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the pixel shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("pixel_shaders\\dilation.psh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble dilation pixel shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the pixel shader
        hr = m_pd3dDevice->CreatePixelShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pDilationPixelShader );
        pCode->Release();
        if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create dilation pixel shader.", "VishShader.exe", MB_OK);
            return hr;
		}
		
	}

    // Create smooth dilation pixel shader
    {
        LPD3DXBUFFER pCode;

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the pixel shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("pixel_shaders\\smooth_dilation.psh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble smooth dilation pixel shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the pixel shader
        hr = m_pd3dDevice->CreatePixelShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pSmoothDilationPixelShader );
        pCode->Release();
        if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create smooth dilation pixel shader.", "VishShader.exe", MB_OK);
            return hr;
		}
		
	}

    // Create smooth shade pixel shader
    {
        LPD3DXBUFFER pCode;

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the pixel shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("pixel_shaders\\smooth_shade.psh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble smooth shade pixel shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the pixel shader
        hr = m_pd3dDevice->CreatePixelShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pSmoothPixelShader );
        pCode->Release();
        if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create smooth shade pixel shader.", "VishShader.exe", MB_OK);
            return hr;
		}
		
	}

    // Create hand edge pixel shader
    {
        LPD3DXBUFFER pCode;

        DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
        dwFlags |= D3DXSHADER_DEBUG;
#endif
        // Assemble the pixel shader from the file

        if( FAILED( hr = D3DXAssembleShaderFromFile( _T("pixel_shaders\\hand_edge.psh"), 
                                                     NULL, NULL, dwFlags, &pCode, NULL ) ) )
        {
			MessageBox(NULL, "Failed to assemble hand edge pixel shader.", "VishShader.exe", MB_OK);
            return hr;
        }

        // Create the pixel shader
        hr = m_pd3dDevice->CreatePixelShader( (DWORD*)pCode->GetBufferPointer(),
                                               &m_pHandEdgePixelShader );
        pCode->Release();
        if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create hand edge pixel shader.", "VishShader.exe", MB_OK);
            return hr;
		}
		
	}


    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RestoreDeviceObjects()
{
	HRESULT hr;

    m_pFont->RestoreDeviceObjects();
    m_pFontSmall->RestoreDeviceObjects();

    // Setup render states
	m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );
//	m_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );

	// Create Render Targets
	{
		// Save Backbuffer info.
		m_pd3dDevice->GetDepthStencilSurface(&m_pBackbufferDepth);
		m_pd3dDevice->GetRenderTarget(0, &m_pBackbufferColor);

		// Create depth buffer surface
		hr = m_pd3dDevice->CreateDepthStencilSurface(static_cast<int>(m_d3dsdBackBuffer.Width / m_fLineWidth), static_cast<int>(m_d3dsdBackBuffer.Height / m_fLineWidth), D3DFMT_D24X8, D3DMULTISAMPLE_NONE, 0, FALSE, &m_pDepthSurfaceA, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create depth surface.", "VishShader.exe", MB_OK);
			return hr;
		}

		// Create render target texture
		hr = m_pd3dDevice->CreateTexture(static_cast<int>(m_d3dsdBackBuffer.Width / m_fLineWidth), static_cast<int>(m_d3dsdBackBuffer.Height / m_fLineWidth), 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &m_pRenderTextureA, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create render texture.", "VishShader.exe", MB_OK);
			return hr;
		}
		// Grab the associated surface
		m_pRenderTextureA->GetSurfaceLevel(0, &m_pRenderSurfaceA);

		// Create render target texture
		hr = m_pd3dDevice->CreateTexture(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &m_pRenderTextureB, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create render texture.", "VishShader.exe", MB_OK);
			return hr;
		}
		// Grab the associated surface
		m_pRenderTextureB->GetSurfaceLevel(0, &m_pRenderSurfaceB);
	}

	// Create 2x2 surface for texture-based lines
	{
		// Create the texture
		hr = m_pd3dDevice->CreateTexture(2, 2, 1, D3DUSAGE_DYNAMIC, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &m_pFakeNormalTexture, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to create fake normal texture.", "VishShader.exe", MB_OK);
			return hr;
		}
		// Grab the associated surface
		m_pFakeNormalTexture->GetSurfaceLevel(0, &m_pFakeNormalSurface);
	}

	// Set texture offset constants
	{
		// first shader size changes based on LineWidth
		float const     kPerTexelWidth  = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Width / m_fLineWidth);
		float const     kPerTexelHeight = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Height / m_fLineWidth);
		float           s               = 0.5f;

		// second offset uses shader thats always the size of backbuffer
		float const     kPerTexelWidth2  = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Width);
		float const     kPerTexelHeight2 = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Height);
		float           s2               = 0.5f;
		float const     rotAngle1       = D3DXToRadian( 0.0f );
		float const     rotAngle2       = rotAngle1 + D3DXToRadian(120.0f); 
		float const     rotAngle3       = rotAngle1 + D3DXToRadian(240.0f); 


		float const OffsetX[4] = {	-s * kPerTexelWidth, 
                                    -s * kPerTexelWidth,  
                                     s * kPerTexelWidth,   
                                     s * kPerTexelWidth  };
		float const OffsetY[4] = {	-s * kPerTexelHeight, 
                                     s * kPerTexelHeight, 
                                     s * kPerTexelHeight, 
                                    -s * kPerTexelHeight };

		float const OffsetBX[4] = {	-s2 * kPerTexelWidth2, 
                                    -s2 * kPerTexelWidth2,  
                                     s2 * kPerTexelWidth2,   
                                     s2 * kPerTexelWidth2  };
		float const OffsetBY[4] = {	-s2 * kPerTexelHeight2, 
                                     s2 * kPerTexelHeight2, 
                                     s2 * kPerTexelHeight2, 
                                    -s2 * kPerTexelHeight2 };
		for (int i = 0; i < 4; ++i)
		{
			D3DXVECTOR4  Offset(OffsetX[i], OffsetY[i], 0.0f, 0.0f);
			D3DXVECTOR4  OffsetB(OffsetBX[i], OffsetBY[i], 0.0f, 0.0f);
    		m_pd3dDevice->SetVertexShaderConstantF(TEXTURE_OFFSET + i, (float *)&Offset, 1);
    		m_pd3dDevice->SetVertexShaderConstantF(TEXTURE_OFFSET_B + i, (float *)&OffsetB, 1);
		}
	}

	float const     kXFix  = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Width);
	float const     kYFix = 1.0f/static_cast<float>(m_d3dsdBackBuffer.Height);


	// Create Quad
	{
		struct QuadVertex
		{
			D3DXVECTOR3 Position;
			D3DXVECTOR2 Tex;
		};
		// create vertex buffer 
		hr = m_pd3dDevice->CreateVertexBuffer( 4 * sizeof(QuadVertex), D3DUSAGE_WRITEONLY, 0, D3DPOOL_DEFAULT, &m_pQuadVB, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Could not create vertex buffer", "VishShader.exe", MB_OK);
			return hr;
		}

		QuadVertex      *pBuff;

		if (m_pQuadVB)
		{
			hr = m_pQuadVB->Lock(0, 4 * sizeof(QuadVertex),(VOID**)&pBuff, 0);
			if (FAILED(hr))
			{
					MessageBox(NULL, "Could not lock vertex buffer", "VishShader.exe", MB_OK);
				return hr;
			}

			for (int i = 0; i < 4; ++i)
			{
				pBuff->Position = D3DXVECTOR3(	(i<2)          ? (-1.0f - kXFix) : (1.0f - kXFix),
												(i==0 || i==3) ? (-1.0f + kYFix) : (1.0f + kYFix),
											  0.0f);
				pBuff->Tex      = D3DXVECTOR2(	(i<2)          ? 0.0f : 1.0f,
												(i==0 || i==3) ? 1.0f : 0.0f 
											  );
				pBuff++;
			}
			m_pQuadVB->Unlock();
		}
	}

	// Load Woman Texture
	{
		// Create the texture
		hr = D3DXCreateTextureFromFile(	m_pd3dDevice, 
										_T("textures\\woman.dds"), 
										&m_pWomanTexture );
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to load woman texture.", "VishShader.exe", MB_OK);
			return hr;
		}
	}

	// Load Ball Texture
	{
		// Create the texture
		hr = D3DXCreateTextureFromFile(	m_pd3dDevice, 
										_T("textures\\beachball.bmp"), 
										&m_pBallTexture );
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to load ball texture.", "VishShader.exe", MB_OK);
			return hr;
		}
	}

	// Load Lines Texture
	{
		// Create the texture
		hr = D3DXCreateTextureFromFile(	m_pd3dDevice, 
										_T("textures\\beachball_lines.dds"), 
										&m_pLinesTexture );
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to load lines texture.", "VishShader.exe", MB_OK);
			return hr;
		}
	}

	// Load Shading Texture
	{
		// Create the texture
		hr = D3DXCreateTextureFromFile(	m_pd3dDevice, 
										_T("textures\\64shade.bmp"), 
										&m_pShadeTextureA );
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to load shade texture.", "VishShader.exe", MB_OK);
			return hr;
		}
	}

	// Load Normal Correcting Cube Map
	{
		// Create the texture
		hr = D3DXCreateCubeTextureFromFile(	m_pd3dDevice, 
										_T("textures//normalize.dds"), 
										&m_pNormalCubeTexture );
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to normal cube texture.", "VishShader.exe", MB_OK);
			return hr;
		}
	}

	// Load Woman Mesh
	{
		DWORD dwNumMaterials;
		LPD3DXMESH pMesh;

	    // Load the mesh from the file
		hr = D3DXLoadMeshFromX(	"meshes//woman.x", D3DXMESH_SYSTEMMEM, 
								m_pd3dDevice, NULL, 
								&m_pWomanD3DXMtrlBuffer, NULL, &dwNumMaterials, 
								&pMesh );
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Could not find woman.x", "VishShader.exe", MB_OK);
			return hr;
		}

		// Optimize the mesh for rendering
		hr = pMesh->OptimizeInplace(D3DXMESHOPT_ATTRSORT, NULL, NULL, NULL, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to optimize mesh.", "VishShader.exe", MB_OK);
			SAFE_RELEASE( pMesh );
			return hr;
		}

		// Clone to proper FVF format
		// | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE1(0)
		hr = pMesh->CloneMeshFVF(D3DXMESH_WRITEONLY | D3DXMESH_NPATCHES, D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE2(0), m_pd3dDevice, &m_pWomanMesh);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to clone mesh.", "VishShader.exe", MB_OK);
			SAFE_RELEASE( pMesh );
			return hr;
		}
		SAFE_RELEASE( pMesh );

		// Get buffers
		m_pWomanMesh->GetVertexBuffer(&m_pWomanVB);
		m_pWomanMesh->GetIndexBuffer(&m_pWomanIB);

		// Extract section information
		hr = m_pWomanMesh->GetAttributeTable(NULL, &m_dwWomanNumSections);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to extract attribute table size.", "VishShader.exe", MB_OK);
			return hr;
		}
		if(m_dwWomanNumSections == 0)
		{
			MessageBox(NULL, "Attribute table has no entries.", "VishShader.exe", MB_OK);
			return E_FAIL;
		}

		m_pWomanAttributes = new D3DXATTRIBUTERANGE[m_dwWomanNumSections];

		hr = m_pWomanMesh->GetAttributeTable(m_pWomanAttributes, &m_dwWomanNumSections);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to extract attribute table.", "VishShader.exe", MB_OK);
			return hr;
		}

	}

	// Load Droid Mesh
	{
		DWORD dwNumMaterials;
		LPD3DXMESH pMesh;

	    // Load the mesh from the file
		hr = D3DXLoadMeshFromX(	"meshes//droid.x", D3DXMESH_SYSTEMMEM, 
								m_pd3dDevice, NULL, 
								&m_pDroidD3DXMtrlBuffer, NULL, &dwNumMaterials, 
								&pMesh );
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Could not find droid.x", "VishShader.exe", MB_OK);
			return hr;
		}

		// Optimize the mesh for rendering
		hr = pMesh->OptimizeInplace(D3DXMESHOPT_ATTRSORT, NULL, NULL, NULL, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to optimize mesh.", "VishShader.exe", MB_OK);
			SAFE_RELEASE( pMesh );
			return hr;
		}

		// Clone to proper FVF format
		// | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE1(0)
		hr = pMesh->CloneMeshFVF(D3DXMESH_WRITEONLY | D3DXMESH_NPATCHES, D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE2(0), m_pd3dDevice, &m_pDroidMesh);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to clone mesh.", "VishShader.exe", MB_OK);
			SAFE_RELEASE( pMesh );
			return hr;
		}
		SAFE_RELEASE( pMesh );

		// Get buffers
		m_pDroidMesh->GetVertexBuffer(&m_pDroidVB);
		m_pDroidMesh->GetIndexBuffer(&m_pDroidIB);

		// Extract section information
		hr = m_pDroidMesh->GetAttributeTable(NULL, &m_dwDroidNumSections);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to extract attribute table size.", "VishShader.exe", MB_OK);
			return hr;
		}
		if(m_dwDroidNumSections == 0)
		{
			MessageBox(NULL, "Attribute table has no entries.", "VishShader.exe", MB_OK);
			return E_FAIL;
		}

		m_pDroidAttributes = new D3DXATTRIBUTERANGE[m_dwDroidNumSections];

		hr = m_pDroidMesh->GetAttributeTable(m_pDroidAttributes, &m_dwDroidNumSections);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to extract attribute table.", "VishShader.exe", MB_OK);
			return hr;
		}

	}

	// Load Ball Mesh
	{
		DWORD dwNumMaterials;
		LPD3DXMESH pMesh;

	    // Load the mesh from the file
		hr = D3DXLoadMeshFromX(	"meshes//beachball.x", D3DXMESH_SYSTEMMEM, 
								m_pd3dDevice, NULL, 
								&m_pBallD3DXMtrlBuffer, NULL, &dwNumMaterials, 
								&pMesh );
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Could not find beachball.x", "VishShader.exe", MB_OK);
			return hr;
		}

		// Optimize the mesh for rendering
		hr = pMesh->OptimizeInplace(D3DXMESHOPT_ATTRSORT, NULL, NULL, NULL, NULL);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to optimize mesh.", "VishShader.exe", MB_OK);
			SAFE_RELEASE( pMesh );
			return hr;
		}

		// Clone to proper FVF format
		// | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE1(0)
		hr = pMesh->CloneMeshFVF(D3DXMESH_WRITEONLY | D3DXMESH_NPATCHES, D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE2(0), m_pd3dDevice, &m_pBallMesh);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to clone mesh.", "VishShader.exe", MB_OK);
			SAFE_RELEASE( pMesh );
			return hr;
		}
		SAFE_RELEASE( pMesh );

		// Get buffers
		m_pBallMesh->GetVertexBuffer(&m_pBallVB);
		m_pBallMesh->GetIndexBuffer(&m_pBallIB);

		// Extract section information
		hr = m_pBallMesh->GetAttributeTable(NULL, &m_dwBallNumSections);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to extract attribute table size.", "VishShader.exe", MB_OK);
			return hr;
		}
		if(m_dwBallNumSections == 0)
		{
			MessageBox(NULL, "Attribute table has no entries.", "VishShader.exe", MB_OK);
			return E_FAIL;
		}

		m_pBallAttributes = new D3DXATTRIBUTERANGE[m_dwBallNumSections];

		hr = m_pBallMesh->GetAttributeTable(m_pBallAttributes, &m_dwBallNumSections);
		if( FAILED(hr) )
		{
			MessageBox(NULL, "Failed to extract attribute table.", "VishShader.exe", MB_OK);
			return hr;
		}

	}

    // Set up the projection matrix
    FLOAT fAspectRatio = (FLOAT)m_d3dsdBackBuffer.Width / (FLOAT)m_d3dsdBackBuffer.Height;
    D3DXMatrixPerspectiveFovLH( &m_matProj, D3DXToRadian(90.0f), fAspectRatio, 2.0f, 2000.0f );

	// set line color
	D3DXVECTOR4 vLineColor(0.0f, 0.0f, 0.2f, 1.0f);
	m_pd3dDevice->SetPixelShaderConstantF(LINE_COLOR, (float *)&vLineColor, 1);

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc:
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InvalidateDeviceObjects()
{
    m_pFont->InvalidateDeviceObjects();
    m_pFontSmall->InvalidateDeviceObjects();

	// Release Textures
	m_pd3dDevice->SetTexture(0, NULL);
	m_pd3dDevice->SetTexture(1, NULL);
	m_pd3dDevice->SetTexture(2, NULL);
	m_pd3dDevice->SetTexture(3, NULL);
	m_pd3dDevice->SetVertexShader(NULL);
	m_pd3dDevice->SetVertexDeclaration(NULL);
	m_pd3dDevice->SetPixelShader(NULL);
    SAFE_RELEASE( m_pRenderTextureA );
    SAFE_RELEASE( m_pRenderTextureB );
    SAFE_RELEASE( m_pFakeNormalTexture );
    SAFE_RELEASE( m_pDepthSurfaceA );
    SAFE_RELEASE( m_pRenderSurfaceA );
    SAFE_RELEASE( m_pRenderSurfaceB );
    SAFE_RELEASE( m_pFakeNormalSurface );
    SAFE_RELEASE( m_pShadeTextureA );
    SAFE_RELEASE( m_pWomanTexture );
    SAFE_RELEASE( m_pBallTexture );
    SAFE_RELEASE( m_pLinesTexture );
    SAFE_RELEASE( m_pNormalCubeTexture );
    SAFE_RELEASE( m_pQuadVB );

	SAFE_RELEASE( m_pWomanIB );
    SAFE_RELEASE( m_pWomanVB );
    SAFE_RELEASE( m_pWomanMesh );
    SAFE_RELEASE( m_pWomanD3DXMtrlBuffer );
	SAFE_DELETE_ARRAY( m_pWomanAttributes );

	SAFE_RELEASE( m_pDroidIB );
    SAFE_RELEASE( m_pDroidVB );
    SAFE_RELEASE( m_pDroidMesh );
    SAFE_RELEASE( m_pDroidD3DXMtrlBuffer );
	SAFE_DELETE_ARRAY( m_pDroidAttributes );

	SAFE_RELEASE( m_pBallIB );
    SAFE_RELEASE( m_pBallVB );
    SAFE_RELEASE( m_pBallMesh );
    SAFE_RELEASE( m_pBallD3DXMtrlBuffer );
	SAFE_DELETE_ARRAY( m_pBallAttributes );

    SAFE_RELEASE( m_pBackbufferColor );
    SAFE_RELEASE( m_pBackbufferDepth );
    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::DeleteDeviceObjects()
{
    m_pFont->DeleteDeviceObjects();
    m_pFontSmall->DeleteDeviceObjects();
    SAFE_RELEASE( m_pNormalEdgeVertexShader );
    SAFE_RELEASE( m_pHandEdgeVertexShader );
    SAFE_RELEASE( m_pToonVertexShader );
    SAFE_RELEASE( m_pQuadVertexShader );
    SAFE_RELEASE( m_pObjectVertexDeclaration );
    SAFE_RELEASE( m_pQuadVertexDeclaration );
    SAFE_RELEASE( m_pQuadPixelShader );
    SAFE_RELEASE( m_pDilationPixelShader );
    SAFE_RELEASE( m_pSmoothDilationPixelShader );
    SAFE_RELEASE( m_pSmoothPixelShader );
    SAFE_RELEASE( m_pHandEdgePixelShader );
 
    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FinalCleanup()
{
    SAFE_DELETE( m_pFont );
    SAFE_DELETE( m_pFontSmall );
    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::ConfirmDevice( D3DCAPS9* pCaps, DWORD dwBehavior,
                                          D3DFORMAT adapterFormat, D3DFORMAT backBufferFormat )
{
    if( (dwBehavior & D3DCREATE_HARDWARE_VERTEXPROCESSING ) ||
        (dwBehavior & D3DCREATE_MIXED_VERTEXPROCESSING ) )
    {
        if( pCaps->VertexShaderVersion < D3DVS_VERSION(1,1) )
            return E_FAIL;
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Message proc function to handle key and menu input
//-----------------------------------------------------------------------------
LRESULT CMyD3DApplication::MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam,
                                    LPARAM lParam )
{
    // Handle key presses
    switch( uMsg )
    {
        case WM_KEYUP:
            m_bKey[wParam] = 0;
            m_bKeyRelease[wParam] = 1;
            break;

        case WM_KEYDOWN:
            m_bKey[wParam] = 1;
            break;

        case WM_COMMAND:
            {
                switch( LOWORD(wParam) )
                {
                case IDM_TOGGLEHELP:
                    m_bShowHelp = !m_bShowHelp;
                    break;
                }
            }
    }

    // Pass remaining messages to default handler
    return CD3DApplication::MsgProc( hWnd, uMsg, wParam, lParam );
}


